@@ -18,8 +18,12 @@ from django.urls import include, path
 
 urlpatterns = [
     path('', include('schema.urls')),
+    path('help/', include('help.urls')),
     path('admin/', admin.site.urls),
     path('accounts/', include('django_registration.backends.activation.urls')),
     path('accounts/', include('django.contrib.auth.urls')),
     path('', include('favicon.urls')),
 ]
+
+
+namespace='author-polls'
\ No newline at end of file
