@@ -74,8 +74,8 @@ class Archive(models.Model):
   location = models.CharField(help_text='Location of this archive', max_length=45, blank=True, null=True)
   storage = models.CharField(choices=ArchiveStorage.choices, help_text='The type of storage used for this archive', max_length=45, blank=True, null=True)
   sealed = models.BooleanField(help_text='Whether or not this archive is sealed (closed to new additions)', default=0)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return self.name
   class Meta:
@@ -149,8 +149,8 @@ class Filter(models.Model):
   attenuation = models.DecimalField(help_text='Attenuation of this filter in decimal stops', max_digits=3, decimal_places=1, blank=True, null=True)
   qty = models.PositiveIntegerField(help_text='Quantity of these filters available', default=1, blank=True, null=True)
   manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this filter')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "%s %smm" % (self.type, str(self.thread))
   class Meta:
@@ -166,9 +166,9 @@ class NegativeSize(models.Model):
   name = models.CharField(help_text='Common name of the negative size (e.g. 35mm, 6x7, etc)', max_length=45, unique=True)
   width = models.DecimalField(help_text='Width of the negative size in mm' ,max_digits=4, decimal_places=1, blank=True, null=True)
   height = models.DecimalField(help_text='Height of the negative size in mm', max_digits=4, decimal_places=1, blank=True, null=True)
-  crop_factor = models.DecimalField(help_text='Crop factor of this negative size', max_digits=4, decimal_places=2, blank=True, null=True)
-  area = models.PositiveIntegerField(help_text='Area of this negative size in sq. mm', blank=True, null=True)
-  aspect_ratio = models.DecimalField(help_text='Aspect ratio of this negative size, expressed as a single decimal (e.g. 3:2 is expressed as 1.5)',max_digits=4, decimal_places=2, blank=True, null=True)
+  crop_factor = models.DecimalField(help_text='Crop factor of this negative size', max_digits=4, decimal_places=2, blank=True, null=True, editable=False)
+  area = models.PositiveIntegerField(help_text='Area of this negative size in sq. mm', blank=True, null=True, editable=False)
+  aspect_ratio = models.DecimalField(help_text='Aspect ratio of this negative size, expressed as a single decimal (e.g. 3:2 is expressed as 1.5)',max_digits=4, decimal_places=2, blank=True, null=True, editable=False)
   def __str__(self):
     return self.name
   # Override save method to calculate some fields
@@ -205,8 +205,8 @@ class Format(models.Model):
 # Table to list all series of cameras and lenses
 class Series(models.Model):
   name = models.CharField(help_text='Name of this collection, e.g. Canon FD SLRs', max_length=45, unique=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return self.name
   class Meta:
@@ -239,8 +239,8 @@ class Flash(models.Model):
   own = models.BooleanField(help_text='Whether we currently own this flash', blank=True, null=True)
   acquired = models.DateField(help_text='Date this flash was acquired', blank=True, null=True)
   cost = MoneyField(help_text='Purchase cost of this flash', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     if self.manufacturer is not None:
       return "%s %s" % (self.manufacturer.name, self.model)
@@ -283,8 +283,8 @@ class Enlarger(models.Model):
   discontinued = models.PositiveIntegerField(help_text='Year in which the enlarger was discontinued', blank=True, null=True)
   cost = MoneyField(help_text='Purchase cost of this enlarger', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
   lost_price = MoneyField(help_text='Sale price of the enlarger', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     if self.manufacturer is not None:
       return "%s %s" % (self.manufacturer.name, self.model)
@@ -394,8 +394,8 @@ class PaperStock(models.Model):
 # Table to catalog photographers
 class Person(models.Model):
   name = models.CharField(help_text='Name of the photographer', max_length=45, unique=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return self.name
   class Meta:
@@ -430,8 +430,8 @@ class Teleconverter(models.Model):
   elements = models.PositiveIntegerField(help_text='Number of optical elements used in this teleconverter', blank=True, null=True)
   groups = models.PositiveIntegerField(help_text='Number of optical groups used in this teleconverter', blank=True, null=True)
   multicoated = models.BooleanField(help_text='Whether this teleconverter is multi-coated', blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     if self.manufacturer is not None:
       return "%s %s" % (self.manufacturer.name, self.model)
@@ -501,8 +501,8 @@ class BulkFilm(models.Model):
   source = models.CharField(help_text='Place where this bulk roll was bought from', max_length=45, blank=True, null=True)
   batch = models.CharField(help_text='Batch code of this bulk roll', max_length=45, blank=True, null=True)
   expiry = models.DateField(help_text='Expiry date of this bulk roll', blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "#%s %s %s" % (self.id_owner, self.filmstock.manufacturer.name, self.filmstock.name)
   class Meta:
@@ -519,8 +519,8 @@ class MountAdapter(models.Model):
   has_optics = models.BooleanField(help_text='Whether this adapter includes optical elements', blank=True, null=True)
   infinity_focus = models.BooleanField(help_text='Whether this adapter allows infinity focus', blank=True, null=True)
   notes = models.CharField(help_text='Freeform notes', max_length=100, blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "%s - %s" % (self.camera_mount, self.lens_mount)
   class Meta:
@@ -535,7 +535,7 @@ class MountAdapter(models.Model):
 class ShutterSpeed(models.Model):
   shutter_speed = models.CharField(help_text='Shutter speed in fractional notation, e.g. 1/250', max_length=10, primary_key=True)
   # field validation: like 1/500 or 2
-  duration = models.DecimalField(help_text='Shutter speed in decimal notation, e.g. 0.04', max_digits=9, decimal_places=5)
+  duration = models.DecimalField(help_text='Shutter speed in decimal notation, e.g. 0.04', max_digits=9, decimal_places=5, editable=False)
   def __str__(self):
     return self.shutter_speed
   def save(self, *args, **kwargs):
@@ -725,6 +725,12 @@ class CameraModel(models.Model):
     Selenium  = ChoiceItem()
     Silicon  = ChoiceItem()
 
+  # Choices for shoe type
+  class ShoeType(DjangoChoices):
+    No_shoe = ChoiceItem()
+    Hot_shoe = ChoiceItem()
+    Cold_shoe = ChoiceItem()
+
   manufacturer = models.ForeignKey(Manufacturer, on_delete=models.CASCADE, blank=True, null=True, help_text='Manufacturer of this camera model')
   model = models.CharField(help_text='The model name of the camera', max_length=45)
   disambiguation = models.CharField(help_text='Distinguishing notes for camera models with the same name', max_length=45, blank=True, null=True)
@@ -738,8 +744,6 @@ class CameraModel(models.Model):
   discontinued = models.PositiveIntegerField(help_text='Year in which the camera model was discontinued', blank=True, null=True)
   body_type = models.CharField(choices=BodyType.choices, max_length=25, blank=True, null=True, help_text='Body type of this camera model')
   weight = models.PositiveIntegerField(help_text='Weight of the camera body (without lens or batteries) in grammes (g)', blank=True, null=True)
-  introduced = models.PositiveIntegerField(help_text='Year in which the camera model was introduced', blank=True, null=True)
-  discontinued = models.PositiveIntegerField(help_text='Year in which the camera model was discontinued', blank=True, null=True)
   negative_size = models.ForeignKey(NegativeSize, on_delete=models.CASCADE, blank=True, null=True, help_text='Size of negative created by this camera')
   shutter_type = models.CharField(choices=ShutterType.choices, max_length=25, blank=True, null=True, help_text='Type of shutter used on this camera model')
   shutter_model = models.CharField(help_text='Model of shutter', max_length=45, blank=True, null=True)
@@ -762,10 +766,9 @@ class CameraModel(models.Model):
   ext_flash = models.BooleanField(verbose_name='External flash', help_text='Whether the camera supports an external flash', blank=True, null=True)
   flash_metering = models.ForeignKey(FlashProtocol, on_delete=models.CASCADE, blank=True, null=True, help_text='Whether this camera model supports flash metering')
   pc_sync = models.BooleanField(verbose_name='PC sync', help_text='Whether the camera has a PC sync socket for flash', blank=True, null=True)
-  hotshoe = models.BooleanField(help_text='Whether the camera has a hotshoe', blank=True, null=True)
-  coldshoe = models.BooleanField(help_text='Whether the camera has a coldshoe or accessory shoe', blank=True, null=True)
+  shoe = models.CharField(choices=ShoeType.choices, max_length=9, blank=True, null=True, help_text='Type of flash/accessory shoe used on this camera model')
   #x_sync = models.ForeignKey(ShutterSpeed, on_delete=models.CASCADE, blank=True, null=True)
-  meter_min_ev = models.PositiveIntegerField(verbose_name='Min EV', help_text='Lowest EV/LV the built-in meter supports', blank=True, null=True)
+  meter_min_ev = models.IntegerField(verbose_name='Min EV', help_text='Lowest EV/LV the built-in meter supports', blank=True, null=True)
   meter_max_ev = models.PositiveIntegerField(verbose_name='Max EV', help_text='Highest EV/LV the built-in meter supports', blank=True, null=True)
   dof_preview = models.BooleanField(verbose_name='DoF preview', help_text='Whether the camera has depth of field preview', blank=True, null=True)
   tripod = models.BooleanField(help_text='Whether the camera has a tripod bush', blank=True, null=True)
@@ -861,8 +864,8 @@ class Accessory(models.Model):
   lost_price = MoneyField(help_text='Sale price of the accessory', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
   camera_model_compatibility = models.ManyToManyField(CameraModel, blank=True)
   lens_model_compatibility = models.ManyToManyField(LensModel, blank=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     if self.manufacturer is not None:
       return "%s %s" % (self.manufacturer.name, self.model)
@@ -906,8 +909,8 @@ class Lens(models.Model):
   source = models.CharField(help_text='Place where the lens was acquired from', max_length=150, blank=True, null=True)
   condition = models.ForeignKey(Condition, on_delete=models.CASCADE, blank=True, null=True, help_text='Condition of this lens')
   condition_notes = models.CharField(help_text='Description of condition', max_length=150, blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "%s %s (#%s)" % (self.lensmodel.manufacturer.name, self.lensmodel.model, self.serial)
   class Meta:
@@ -960,8 +963,8 @@ class Camera(models.Model):
   condition = models.ForeignKey(Condition, on_delete=models.CASCADE, blank=True, null=True, help_text='Condition of this camera')
   condition_notes = models.CharField(help_text='Description of condition', max_length=150, blank=True, null=True)
   display_lens = models.OneToOneField(Lens, on_delete=models.CASCADE, blank=True, null=True, help_text='Lens that this camera should be displayed with', related_name='display_camera')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "%s %s (#%s)" % (self.cameramodel.manufacturer.name, self.cameramodel.model, self.serial)
   class Meta:
@@ -1022,8 +1025,8 @@ class Film(models.Model):
   price = MoneyField(help_text='Price paid for this film', max_digits=12, decimal_places=2, blank=True, null=True, default_currency='GBP')
   processed_by = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Person or place that processed this film')
   archive = models.ForeignKey(Archive, on_delete=models.CASCADE, blank=True, null=True, help_text='Archive that this film is stored in')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "#%i %s" % (self.id_owner, self.title)
   class Meta:
@@ -1071,8 +1074,8 @@ class Negative(models.Model):
   exposure_program = models.ForeignKey(ExposureProgram, on_delete=models.CASCADE, blank=True, null=True, help_text='Exposure program used when taking the image')
   photographer = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Photographer who took the negative')
   copy_of = models.ForeignKey('self', on_delete=models.CASCADE, blank=True, null=True, related_name='copy', help_text='Negative that this was duplicated from')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "%s/%s %s" % (self.film.pk, self.frame, self.caption)
   class Meta:
@@ -1134,8 +1137,8 @@ class Print(models.Model):
   notes = models.CharField(help_text='Freeform notes about this print, e.g. dodging, burning & complex toning', max_length=200, blank=True, null=True)
   archive = models.ForeignKey(Archive, on_delete=models.CASCADE, blank=True, null=True, help_text='Archive that this print is stored in')
   printer = models.ForeignKey(Person, on_delete=models.CASCADE, blank=True, null=True, help_text='Person who made this print')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "#%i" % (self.id_owner)
   class Meta:
@@ -1174,8 +1177,8 @@ class Repair(models.Model):
   date = models.DateField(help_text='The date of the repair', blank=True, null=True)
   summary = models.CharField(help_text='Brief summary of the repair', max_length=100)
   detail = models.CharField(help_text='Longer description of the repair', max_length=500, blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "#%i" % (self.id_owner)
   class Meta:
@@ -1195,8 +1198,8 @@ class Scan(models.Model):
   colour = models.BooleanField(help_text='Whether this is a colour image', blank=True, null=True)
   width = models.PositiveIntegerField(help_text='Width of the scanned image in pixels', blank=True, null=True)
   height = models.PositiveIntegerField(help_text='Height of the scanned image in pixels', blank=True, null=True)
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return self.filename
   def clean(self):
@@ -1222,8 +1225,8 @@ class Order(models.Model):
   printed = models.BooleanField(help_text='Whether the print has been made', blank=True, null=True)
   print = models.ForeignKey(Print, on_delete=models.CASCADE, blank=True, null=True, help_text='Print that was made to fulfil this order')
   recipient = models.ForeignKey(Person, on_delete=models.CASCADE, help_text='Person who placed this order')
-  owner = CurrentUserField()
-  id_owner = AutoSequenceField(unique_with='owner')
+  owner = CurrentUserField(editable=False)
+  id_owner = AutoSequenceField(unique_with='owner', editable=False)
   def __str__(self):
     return "#%i" % (self.id_owner)
   class Meta:
